==> mydns_0.9.2.orig.tar.gz: mydns-0.9.2/src/lib/soa.c <==
/**************************************************************************************************
	$Header: /pub/cvsroot/mydns/src/lib/soa.c,v 1.32 2002/09/20 21:15:30 bboy Exp $

	Copyright (C) 2002  Don Moore <bboy@bboy.net>

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at Your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
**************************************************************************************************/

#include "mydns.h"

char mydns_soa_table_name[PATH_MAX] = MYDNS_SOA_TABLE;

/* Optional columns */
int mydns_soa_use_active = 0;
int mydns_soa_use_xfer = 0;


/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
	MYDNS_SOA_COUNT
	Returns the number of zones in the soa table.
+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
u_int32_t
mydns_soa_count(mysql)
	MYSQL *mysql;
{
	MYSQL_RES *res = NULL;
	MYSQL_ROW row;
	char query[512];
	size_t querylen;
	u_int32_t rv = 0;

	querylen = snprintf(query, sizeof(query), "SELECT COUNT(*) FROM %s", mydns_soa_table_name);
	if (mysql_real_query(mysql, query, querylen) || !(res = mysql_store_result(mysql)))
		return (-1);
	if ((row = mysql_fetch_row(res)))
		rv = (u_int32_t)strtoul(row[0], (char **)NULL, 10);
	mysql_free_result(res);
	return (rv);
}
/*--- mydns_soa_count() -------------------------------------------------------------------------*/


/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
	MYDNS_SET_SOA_TABLE_NAME
+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
void
mydns_set_soa_table_name(name)
	char *name;
{
	if (!name)
		strncpy(mydns_soa_table_name, MYDNS_SOA_TABLE, sizeof(mydns_soa_table_name)-1);
	else
		strncpy(mydns_soa_table_name, name, sizeof(mydns_soa_table_name)-1);
}
/*--- mydns_set_soa_table_name() ----------------------------------------------------------------*/


/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
	MYDNS_PARSE_SOA
+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
inline MYDNS_SOA *
mydns_parse_soa(row)
	MYSQL_ROW row;
{
	MYDNS_SOA *rv;

	if ((rv = (MYDNS_SOA *)calloc(1, sizeof(MYDNS_SOA))))
	{
		rv->next = NULL;

		rv->id = atou(row[0]);
		strncpy(rv->origin, row[1], sizeof(rv->origin)-1);
		strncpy(rv->ns, row[2], sizeof(rv->ns)-1);
		if (!rv->ns[0])
			snprintf(rv->ns, sizeof(rv->ns), "ns.%s", rv->origin);
		strncpy(rv->mbox, row[3], sizeof(rv->mbox)-1);
		if (!rv->mbox[0])
			snprintf(rv->mbox, sizeof(rv->mbox), "hostmaster.%s", rv->origin);
		rv->serial = atou(row[4]);
		rv->refresh = atou(row[5]);
		rv->retry = atou(row[6]);
		rv->expire = atou(row[7]);
		rv->minimum = atou(row[8]);
		rv->ttl = atou(row[9]);

		/* Make sure TTL for SOA is at least the minimum */
		/* XXX: Update this in the database, maybe */
		if (rv->ttl < rv->minimum)
			rv->ttl = rv->minimum;
	}
	return (rv);
}
/*-----------------------------------------------------------------------------------------------*/


/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
	MYDNS_SOA_DUP
	Create a duplicate copy of the record.
+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
MYDNS_SOA *
mydns_soa_dup(start, recurse)
	MYDNS_SOA *start;
	int recurse;
{
	register MYDNS_SOA *first = NULL, *last = NULL, *soa, *s, *tmp;

	for (s = start; s; s = tmp)
	{
		tmp = s->next;

		if ((soa = (MYDNS_SOA *)malloc(sizeof(MYDNS_SOA))))
		{
			soa->id = s->id;
			strncpy(soa->origin, s->origin, sizeof(soa->origin)-1);
			strncpy(soa->ns, s->ns, sizeof(soa->ns)-1);
			strncpy(soa->mbox, s->mbox, sizeof(soa->mbox)-1);
			soa->serial = s->serial;
			soa->refresh = s->refresh;
			soa->retry = s->retry;
			soa->expire = s->expire;
			soa->minimum = s->minimum;
			soa->ttl = s->ttl;
			soa->next = NULL;
			if (recurse)
			{
				if (!first) first = soa;
				if (last) last->next = soa;
				last = soa;
			}
			else
				return (soa);
		}
	}
	return (first);
}
/*--- mydns_dup_soa() ---------------------------------------------------------------------------*/


/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
	MYDNS_SOA_SIZE
+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
inline size_t
mydns_soa_size(first)
	MYDNS_SOA *first;
{
	register MYDNS_SOA *p;
	register size_t size = 0;

	for (p = first; p; p = p->next)
		size += sizeof(MYDNS_SOA) + strlen(p->origin) + strlen(p->ns) + strlen(p->mbox);

	return (size);
}
/*-----------------------------------------------------------------------------------------------*/



/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
	_MYDNS_SOA_FREE
	Frees the pointed-to structure.	Don't call this function directly, call the macro.
+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
inline void
_mydns_soa_free(first)
	MYDNS_SOA *first;
{
	register MYDNS_SOA *p, *tmp;

	for (p = first; p; p = tmp)
	{
		tmp = p->next;

		free(p);
	}
}
/*-----------------------------------------------------------------------------------------------*/


/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
	MYDNS_SOA_LOAD
	Returns 0 on success or nonzero if an error occurred.
+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
int
mydns_soa_load(mysql, rptr, origin)
	MYSQL	*mysql;					/* MySQL connection */
	MYDNS_SOA **rptr;				/* Result list will be returned here */
	u_char *origin;				/* The origin to look up in the `soa' table */
{
	MYDNS_SOA *first = NULL, *last = NULL;
	size_t	 originlen, querylen;
	u_char	 query[DNS_QUERYBUFSIZ];
	u_char	 esc[DNS_MAXNAMELEN + DNS_MAXNAMELEN + 1];
	MYSQL_RES *res;
	MYSQL_ROW row;

	if (rptr) *rptr = NULL;
	MYDNS_CLEARERR(mysql);

	/* Verify args */
	if (!mysql || !origin || !rptr)
	{
		errno = EINVAL;
		return (-1);
	}

	/* Escape `origin' */
	originlen = strlen(origin);
	if (originlen + originlen + 1 > sizeof(esc))
	{
		errno = EINVAL;
		return (-1);
	}
	mysql_real_escape_string(mysql, esc, origin, originlen);

	/* Construct query */
	if (mydns_soa_use_active)
		querylen = snprintf(query, sizeof(query),
			"SELECT "MYDNS_SOA_FIELDS" FROM %s WHERE origin='%s' AND active=1",
			mydns_soa_table_name, esc);
	else
		querylen = snprintf(query, sizeof(query),
			"SELECT "MYDNS_SOA_FIELDS" FROM %s WHERE origin='%s'", mydns_soa_table_name, esc);

	/* Submit query */
	if (mysql_real_query(mysql, query, querylen) || !(res = mysql_store_result(mysql)))
		return (-1);

	/* Add results to list */
	while ((row = mysql_fetch_row(res)))
	{
		MYDNS_SOA *new;

		if (!(new = mydns_parse_soa(row)))
		{
			mysql_free_result(res);
			return (-1);
		}
		if (!first) first = new;
		if (last) last->next = new;
		last = new;
	}
	*rptr = first;
	mysql_free_result(res);
	return (0);
}
/*-----------------------------------------------------------------------------------------------*/


/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
	MYDNS_SOA_MAKE
	Generates a SOA record for requests with missing SOA (i.e. in-addr.arpa zones).
	Returns 0 on success or nonzero if an error occurred.
+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
int
mydns_soa_make(mysql, rptr, origin, hostname)
	MYSQL *mysql;
	MYDNS_SOA **rptr;				/* Result list will be returned here */
	u_char *origin;				/* The origin to look up in the `soa' table */
	u_char *hostname;				/* This machine's hostname */
{
	MYDNS_SOA *soa = NULL;
	u_char ip[4], query[DNS_QUERYBUFSIZ];
	MYSQL_RES *res;
	MYSQL_ROW row;

	if (rptr) *rptr = NULL;
	MYDNS_CLEARERR(mysql);
	if (!mysql || !origin || !rptr)
	{
		errno = EINVAL;
		return (-1);
	}

	/* Make sure there is at least one PTR record */
	switch (mydns_extract_arpa(origin, ip))
	{
		case 3:
			snprintf(query, sizeof(query),
				"SELECT COUNT(*) FROM %s WHERE "
				"ip >= INET_ATON('%d.%d.%d.0') AND ip <= INET_ATON('%d.%d.%d.255')%s",
				mydns_ptr_table_name, ip[2], ip[1], ip[0], ip[2], ip[1], ip[0],
				mydns_ptr_use_active ? " AND active=1" : "");
			break;

		case 2:
			snprintf(query, sizeof(query),
				"SELECT COUNT(*) FROM %s WHERE "
				"ip >= INET_ATON('%d.%d.0.0') AND ip <= INET_ATON('%d.%d.255.255')%s",
				mydns_ptr_table_name, ip[1], ip[0], ip[1], ip[0],
				mydns_ptr_use_active ? " AND active=1" : "");
			break;

		case 1:
			snprintf(query, sizeof(query),
				"SELECT COUNT(*) FROM %s WHERE "
				"ip >= INET_ATON('%d.0.0.0') AND ip <= INET_ATON('%d.255.255.255')%s",
				mydns_ptr_table_name, ip[0], ip[0],
				mydns_ptr_use_active ? " AND active=1" : "");
			break;

		default:
			return (-1);		/* Invalid format for zone */
	}
	if (mysql_real_query(mysql, query, strlen(query)) || !(res = mysql_store_result(mysql)))
		return (-1);
	if (!mysql_num_rows(res) || !(row = mysql_fetch_row(res)))
	{
		mysql_free_result(res);
		return (0);
	}
	if (!atou(row[0]))
	{
		mysql_free_result(res);
		return (0);
	}
	mysql_free_result(res);

	/* Create fake SOA record */
	if (!(soa = (MYDNS_SOA *)calloc(1, sizeof(MYDNS_SOA))))
		return (-1);

	soa->id = 0;
	strncpy(soa->origin, origin, sizeof(soa->origin)-1);
	snprintf(soa->ns, sizeof(soa->ns), "%s.", hostname);
	snprintf(soa->mbox, sizeof(soa->mbox), "hostmaster.%s.", hostname);
	soa->serial = time(NULL);
	soa->refresh = DNS_DEFAULT_REFRESH;
	soa->retry = DNS_DEFAULT_RETRY;
	soa->expire = DNS_DEFAULT_EXPIRE;
	soa->minimum = DNS_DEFAULT_MINIMUM;
	soa->ttl = DNS_DEFAULT_TTL;
	soa->next = NULL;

	*rptr = soa;

	return (0);
}
/*-----------------------------------------------------------------------------------------------*/

/* vi:set ts=3: */

==> mydns_0.10.1.orig.tar.gz: mydns-0.10.1/src/lib/soa.c <==
/**************************************************************************************************
	$Header: /pub/cvsroot/mydns/src/lib/soa.c,v 1.53 2003/08/05 13:27:22 bboy Exp $

	Copyright (C) 2002-2003  Don Moore <bboy@bboy.net>

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at Your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
**************************************************************************************************/

#include "mydns.h"

char mydns_soa_table_name[PATH_MAX] = MYDNS_SOA_TABLE;
char *mydns_soa_where_clause = NULL;

/* Optional columns */
int mydns_soa_use_active = 0;
int mydns_soa_use_xfer = 0;


/**************************************************************************************************
	MYDNS_SOA_COUNT
	Returns the number of zones in the soa table.
**************************************************************************************************/
long
mydns_soa_count(SQL *sql)
{
	return sql_count(sql, "SELECT COUNT(*) FROM %s", mydns_soa_table_name);
}
/*--- mydns_soa_count() -------------------------------------------------------------------------*/


/**************************************************************************************************
	MYDNS_SET_SOA_TABLE_NAME
**************************************************************************************************/
void
mydns_set_soa_table_name(char *name)
{
	if (!name)
		strncpy(mydns_soa_table_name, MYDNS_SOA_TABLE, sizeof(mydns_soa_table_name)-1);
	else
		strncpy(mydns_soa_table_name, name, sizeof(mydns_soa_table_name)-1);
}
/*--- mydns_set_soa_table_name() ----------------------------------------------------------------*/


/**************************************************************************************************
	MYDNS_SET_SOA_WHERE_CLAUSE
**************************************************************************************************/
void
mydns_set_soa_where_clause(char *where)
{
	if (where && strlen(where))
	{
		if (!(mydns_soa_where_clause = strdup(where)))
			Errx("out of memory");
	}
}
/*--- mydns_set_soa_where_clause() --------------------------------------------------------------*/


/**************************************************************************************************
	MYDNS_PARSE_SOA
**************************************************************************************************/
#if !PROFILING
inline
#endif
MYDNS_SOA *
mydns_parse_soa(SQL_ROW row)
{
	MYDNS_SOA *rv;
	int len;

	if ((rv = (MYDNS_SOA *)malloc(sizeof(MYDNS_SOA))))
	{
		rv->next = NULL;

		rv->id = atou(row[0]);
		strncpy(rv->origin, row[1], sizeof(rv->origin)-1);
		strncpy(rv->ns, row[2], sizeof(rv->ns)-1);
		if (!rv->ns[0])
			snprintf(rv->ns, sizeof(rv->ns), "ns.%s", rv->origin);
		strncpy(rv->mbox, row[3], sizeof(rv->mbox)-1);
		if (!rv->mbox[0])
			snprintf(rv->mbox, sizeof(rv->mbox), "hostmaster.%s", rv->origin);
		rv->serial = atou(row[4]);
		rv->refresh = atou(row[5]);
		rv->retry = atou(row[6]);
		rv->expire = atou(row[7]);
		rv->minimum = atou(row[8]);
		rv->ttl = atou(row[9]);

		/* If 'ns' or 'mbox' don't end in a dot, append the origin */
		len = strlen(rv->ns);
		if (rv->ns[len-1] != '.')
		{
			strncat(rv->ns, ".", sizeof(rv->ns) - len - 1);
			strncat(rv->ns, rv->origin, sizeof(rv->ns) - len - 2);
		}
		len = strlen(rv->mbox);
		if (rv->mbox[len-1] != '.')
		{
			strncat(rv->mbox, ".", sizeof(rv->mbox) - len - 1);
			strncat(rv->mbox, rv->origin, sizeof(rv->mbox) - len - 2);
		}

		/* Make sure TTL for SOA is at least the minimum */
		if (rv->ttl < rv->minimum)
			rv->ttl = rv->minimum;
	}
	return (rv);
}
/*--- mydns_parse_soa() -------------------------------------------------------------------------*/


/**************************************************************************************************
	MYDNS_SOA_DUP
	Create a duplicate copy of the record.
**************************************************************************************************/
MYDNS_SOA *
mydns_soa_dup(MYDNS_SOA *start, int recurse)
{
	register MYDNS_SOA *first = NULL, *last = NULL, *soa, *s, *tmp;

	for (s = start; s; s = tmp)
	{
		tmp = s->next;

		if ((soa = (MYDNS_SOA *)malloc(sizeof(MYDNS_SOA))))
		{
			soa->id = s->id;
			strncpy(soa->origin, s->origin, sizeof(soa->origin)-1);
			strncpy(soa->ns, s->ns, sizeof(soa->ns)-1);
			strncpy(soa->mbox, s->mbox, sizeof(soa->mbox)-1);
			soa->serial = s->serial;
			soa->refresh = s->refresh;
			soa->retry = s->retry;
			soa->expire = s->expire;
			soa->minimum = s->minimum;
			soa->ttl = s->ttl;
			soa->next = NULL;
			if (recurse)
			{
				if (!first) first = soa;
				if (last) last->next = soa;
				last = soa;
			}
			else
				return (soa);
		}
	}
	return (first);
}
/*--- mydns_soa_dup() ---------------------------------------------------------------------------*/


/**************************************************************************************************
	MYDNS_SOA_SIZE
**************************************************************************************************/
#if !PROFILING
inline
#endif
size_t
mydns_soa_size(MYDNS_SOA *first)
{
	register MYDNS_SOA *p;
	register size_t size = 0;

	for (p = first; p; p = p->next)
		size += sizeof(MYDNS_SOA);

	return (size);
}
/*--- mydns_soa_size() --------------------------------------------------------------------------*/


/**************************************************************************************************
	_MYDNS_SOA_FREE
	Frees the pointed-to structure.	Don't call this function directly, call the macro.
**************************************************************************************************/
#if !PROFILING
inline
#endif
void
_mydns_soa_free(MYDNS_SOA *first)
{
	register MYDNS_SOA *p, *tmp;

	for (p = first; p; p = tmp)
	{
		tmp = p->next;
		Free(p);
	}
}
/*--- mydns_soa_free() --------------------------------------------------------------------------*/


/**************************************************************************************************
	MYDNS_SOA_LOAD
	Returns 0 on success or nonzero if an error occurred.
**************************************************************************************************/
int
mydns_soa_load(SQL *sql, MYDNS_SOA **rptr, char *origin)
{
	MYDNS_SOA		*first = NULL, *last = NULL;
	size_t			querylen;
	char				query[DNS_QUERYBUFSIZ];
	SQL_RES			*res;
	SQL_ROW			row;
	register char	*c;

	if (rptr) *rptr = NULL;

	/* Verify args */
	if (!sql || !origin || !rptr)
	{
		errno = EINVAL;
		return (-1);
	}

	/* We're not escaping 'origin', so check it for illegal type chars */
	for (c = origin; *c; c++)
		if (SQL_BADCHAR(*c))
			return (0);

	/* Construct query */
	if (mydns_soa_where_clause)
		querylen = snprintf(query, sizeof(query),
			"SELECT "MYDNS_SOA_FIELDS"%s FROM %s WHERE origin='%s' AND %s",
			(mydns_soa_use_active ? ",active" : ""),
			mydns_soa_table_name, origin, mydns_soa_where_clause);
	else
		querylen = snprintf(query, sizeof(query),
			"SELECT "MYDNS_SOA_FIELDS"%s FROM %s WHERE origin='%s'",
			(mydns_soa_use_active ? ",active" : ""),
			mydns_soa_table_name, origin);

	/* Submit query */
	if (!(res = sql_query(sql, query, querylen)))
		return (-1);

	/* Add results to list */
	while ((row = sql_getrow(res)))
	{
		MYDNS_SOA *new;

		if (mydns_soa_use_active && row[MYDNS_SOA_NUMFIELDS] && !GETBOOL(row[MYDNS_SOA_NUMFIELDS]))
			continue;

		if (!(new = mydns_parse_soa(row)))
		{
			sql_free(res);
			return (-1);
		}
		if (!first) first = new;
		if (last) last->next = new;
		last = new;
	}

	*rptr = first;
	sql_free(res);
	return (0);
}
/*--- mydns_soa_load() --------------------------------------------------------------------------*/

/* vi:set ts=3: */
